/*
 *	ff7decode.c
 *		Copyright (C)2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "ff7decode.h"

/* t@Cit@^W[VII̕R[h\(VOoCg) */
static const char *tbl_charcode[] =
{
	"o", "", "r", "", "u", "", "x", "", "{", "", "K", "", "M", "", "O", "", 
	"Q", "", "S", "", "U", "", "W", "", "Y", "", "[", "", "]", "", "_", "", 
	"a", "", "d", "", "f", "", "h", "", "", "p", "", "s", "", "v", "", "y", 
	"", "|", "", "0",  "1",  "2",  "3",  "4",  "5",  "6",  "7",  "8",  "9",  "A", "B", " ", 
	"n", "", "q", "", "t", "", "w", "", "z", "", "J", "", "L", "", "N", "", 
	"P", "", "R", "", "T", "", "V", "", "X", "", "Z", "", "\", "", "^", "", 
	"`", "", "c", "", "e", "", "g", "", "E", "", "A", "", "C", "", "G", "", 
	"I", "", "i", "", "j", "", "k", "", "l", "", "m", "", "}", "", "~", "", 
	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", 
	"", "", "", "", "", "", "", "", "", "", "", "", "b", "", "", "", 
	"", "", "", "", "@", "", "B", "", "D", "", "F", "", "H", "", "!",  "?", 
	"u", "v", ".",  "+",  "A",  "B",  "C",  "D",  "E",  "F",  "G",  "H",  "I",  "J",  "K",  "L", 
	"M",  "N",  "O",  "P",  "Q",  "R",  "S",  "T",  "U",  "V",  "W",  "X",  "Y",  "Z",  "E", "*", 
	"[", "`", "c", "%",  "/",  ":",  "&",  "y", "z", " ",  "", "", "", "u", "v", "(", 
	")",  "-",  "=",  "", "", "", "XIII", "", "", "", "", "", "", "", "", "", 
	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", " ",
};

char *dec_string(char *ptr_dstbuf, const unsigned char *ptr_srcbuf, size_t dstbufsize, size_t srcbufsize, int flag_multibyte)
{
	int i;
	char *buf_dstbuf = NULL;
	char *ptr = NULL;
	const size_t len_buf = srcbufsize * 8;

	assert(sizeof(tbl_charcode) / sizeof(tbl_charcode[0]) == 256);

	/* `FbN */
	if (ptr_dstbuf == NULL)
	{
		assert(0);
		return NULL;
	}
	if (ptr_srcbuf == NULL || srcbufsize == 0 || dstbufsize == 0)
	{
		assert(0);
		return ptr_dstbuf;
	}

	/* ꎞobt@mۂ */
	if ((buf_dstbuf = malloc(len_buf)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* fR[h */
	memset(buf_dstbuf, '\0', sizeof(len_buf));
	if (!flag_multibyte)
	{
		for (i = 0; i < (int)srcbufsize; i++)
			strcat(buf_dstbuf, tbl_charcode[ptr_srcbuf[i]]);
	}
	else
	{
		/* }`oCg͖T|[g */
	}

	/* ̋󔒂 */
	while ((ptr = strrchr(buf_dstbuf, ' ')) != NULL && *(ptr + 1) == '\0')
		*ptr = '\0';

	/* ꎞobt@̓eo͂ */
	strncpy(ptr_dstbuf, buf_dstbuf, dstbufsize - 1);
	ptr_dstbuf[dstbufsize - 1] = '\0';

	free(buf_dstbuf);
	buf_dstbuf = NULL;

	return ptr_dstbuf;
}

int dec_file(const char *ptr_readfilename, const char *ptr_writefilename, int flag_multibyte)
{
	int ch;
	FILE *fp_read = NULL;
	FILE *fp_write = NULL;

	assert(sizeof(tbl_charcode) / sizeof(tbl_charcode[0]) == 256);

	/* `FbN */
	if (ptr_readfilename == NULL || *ptr_readfilename == '\0' || ptr_writefilename == NULL || *ptr_writefilename == '\0')
	{
		assert(0);
		return 0;
	}

	/* fR[ht@CJ */
	if ((fp_read = fopen(ptr_readfilename, "rb")) == NULL)
		return 0;

	/* fR[hʂo͂t@CJ */
	if ((fp_write = fopen(ptr_writefilename, "w")) == NULL)
	{
		fclose(fp_read);
		fp_read = NULL;
		return 0;
	}

	/* fR[h */
	if (!flag_multibyte)
	{
		while ((ch = fgetc(fp_read)) != EOF)
		{
			if (ch == 0x00)	/* Kxɉs邽 */
				fputc('\n', fp_write);
			else
				fputs(tbl_charcode[ch], fp_write);
		}
	}
	else
	{
		/* }`oCg͖T|[g */
	}

	fclose(fp_read);
	fclose(fp_write);
	fp_read  = NULL;
	fp_write = NULL;

	return 1;
}
