/*
 *	ff7lovechecker.c
 *		Copyright (C)2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "errmes.h"
#include "memcard.h"
#include "ff7savedata.h"

#define STR_TOO_FEW_ARGUMENTS		"Too few arguments"
#define STR_INVALID_OPTION			"Invalid option"
#define STR_INVALID_SAVEDATA_FILE	"Invalid save-data file"

/* 쌠\L */
static const char *str_copyright = "ff7lovechecker - Copyright (C)2003 Shingo Endo <s-endo@yk.rim.or.jp>";

int main(int argc, char **argv)
{
	int i;
	int ret = EXIT_SUCCESS;
	int flag_states  = 0;
	int flag_title   = 0;
	int flag_usage   = 0;
	int flag_invalid = 0;
	char str_srcfilename[1024] = "";
	char str_dstfilename[1024] = "";
	FILE *fp_src = NULL;
	FILE *fp_dst = NULL;
	const unsigned char scheader_ff7[] =
	{
		0x53, 0x43,	/* ʎq("SC") */
		0x11,		/* ACR̗LE(1) */
		0x01,		/* gpubN(1ubN) */
					/* ^Cg(Shift_JIS) */
		0x82, 0x65, 0x82, 0x65, 0x82, 0x56,
	};

	errno = 0;
	err_setprogname(argv[0]);

	/* R}hC͂ */
	for (i = 1; i < argc; i++)
	{
		if (argv[i] == NULL)
			continue;

		if (*argv[i] == '-')
		{
			const char *ptr = argv[i];

			while (*++ptr != '\0')
			{
				switch (tolower(*ptr))
				{
				case 't':	/* Z[uf[^̃^Cg̕\ */
					flag_title = 1;
					break;
				case 's':	/* Xe[^X̕\ */
					flag_states = 1;
					break;
				case 'h':	/* gp@̕\ */
					flag_usage = 1;
					break;
				default:
					flag_invalid = 1;
					break;
				}
			}
		}
		else
		{
			if (*str_srcfilename == '\0')
			{
				strncpy(str_srcfilename, argv[i], sizeof(str_srcfilename));
				str_srcfilename[sizeof(str_srcfilename) - 1] = '\0';
			}
			else if (*str_dstfilename == '\0')
			{
				strncpy(str_dstfilename, argv[i], sizeof(str_dstfilename));
				str_dstfilename[sizeof(str_dstfilename) - 1] = '\0';
			}
		}
	}

	/* gp@\ */
	if (flag_usage)
	{
		puts(str_copyright);
		printf("%s: [option] src-filename [dst-filename]\n", err_getprogname());
		printf("\t-t\tprint save-data's title\n");
		printf("\t-s\tprint states of each characters\n");
		printf("\t-h\tdisplay this help and exit\n");
		printf("\n");

		exit(EXIT_SUCCESS);
	}

	/* R}hC`FbN */
	if (flag_invalid)
		err_printf(STR_INVALID_OPTION);
	if (*str_srcfilename == '\0')
		err_printf(STR_TOO_FEW_ARGUMENTS);

	/* ̓t@CI[v */
	if ((fp_src = fopen(str_srcfilename, "rb")) == NULL)
		err_printf("%s:", str_srcfilename);

	/* o̓t@CI[v */
	if (*str_dstfilename == '\0')
		fp_dst = stdout;
	else
	{
		if ((fp_dst = fopen(str_dstfilename, "w")) == NULL)
			err_printf("%s:", str_dstfilename);
	}

	/* ̓t@Cɑ݂邷ׂĂFFVII[J[hf[^ēeo͂ */
	while (!mc_ptnseek(fp_src, scheader_ff7, sizeof(scheader_ff7)))
	{
		unsigned char *buf = NULL;
		savedata_t *buf_savedata  = NULL;

		/* [J[hf[^i[obt@mۂ */
		if ((buf = malloc(MC_BLOCK_SIZE)) == NULL)
			err_printf(STR_FATAL_ERROR_IN_MALLOC);
		memset(buf, 0x00, MC_BLOCK_SIZE);

		/* [J[hf[^ǂݍ */
		if (fread(buf, 1, MC_BLOCK_SIZE, fp_src) < MC_BLOCK_SIZE && ferror(fp_src))
			err_printf(STR_FATAL_ERROR_IN_FREAD);

		/* [J[hf[^́E͂ */
		buf_savedata = sav_readdata(buf);

		/* ͌ʂo͂ */
		if (buf_savedata == NULL)
		{
			wrn_printf(STR_INVALID_SAVEDATA_FILE " - %s", str_srcfilename);
			ret = EXIT_FAILURE;
		}
		else
		{
			if (flag_title)
				sav_writetitle(fp_dst, buf_savedata);

			sav_writelove(fp_dst, buf_savedata);

			if (flag_states)
				sav_writestatus(fp_dst, buf_savedata);

			fputc('\n', fp_dst);
		}

		/* obt@ */
		free(buf);
		buf = NULL;
	}

	/* t@CN[Y */
	fclose(fp_src);
	if (fp_dst != stdout)
		fclose(fp_dst);

	return ret;
}
