/*
 *	memcard.c
 *		Copyright (C)2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include "errmes.h"
#include "memcard.h"

int mc_ptnseek(FILE *stream, const unsigned char *ptr_matchptn, size_t ptnsize)
{
	int i;
	int flag_success = 0;
	long file_offset;
	long file_pos;
	long file_tail;
	unsigned char *buf = NULL;
	unsigned char *ptr_buf = NULL;
	int tbl_skip[256];
	const size_t bufsize = (ptnsize * 16 > 1024) ? ptnsize * 16 : 1024;

	/* `FbN */
	if (stream == NULL || ptr_matchptn == NULL || ptnsize == 0)
	{
		assert(0);
		return 0;
	}

	/* t@C̏I[̃ItZbg擾 */
	file_offset = file_pos = ftell(stream);
	if (fseek(stream, 0, SEEK_END))
		err_printf(STR_FATAL_ERROR_IN_FSEEK);
	file_tail = ftell(stream);
	if (fseek(stream, file_offset, SEEK_SET))
		err_printf(STR_FATAL_ERROR_IN_FSEEK);

	/* p̃XLbve[u쐬 */
	for (i = 0; i < sizeof(tbl_skip) / sizeof(tbl_skip[0]); i++)
		tbl_skip[i] = (int)ptnsize;
	for (i = 0; i < (int)ptnsize - 1; i++)
		tbl_skip[ptr_matchptn[i]] = (int)ptnsize - i - 1;

	/* ꎞobt@mۂ */
	if ((buf = malloc(bufsize)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);
	ptr_buf = buf;

	/* Jn */
	if (fread(buf, 1, ptnsize, stream) < ptnsize && ferror(stream))
		err_printf(STR_FATAL_ERROR_IN_FREAD);
	ptr_buf += ptnsize;

	while (file_pos < file_tail)
	{
		if (*(ptr_buf - 1) == ptr_matchptn[ptnsize - 1])
		{
			if (memcmp(ptr_buf - ptnsize, ptr_matchptn, ptnsize) == 0)
			{
				if (fseek(stream, -(int)ptnsize, SEEK_CUR))
					err_printf(STR_FATAL_ERROR_IN_FSEEK);
				flag_success = 1;
				break;
			}
		}

		if (ptr_buf - buf + tbl_skip[*(ptr_buf - 1)] >= (ptrdiff_t)bufsize - 8)
		{
			memmove(buf, ptr_buf - ptnsize, ptnsize);
			ptr_buf = &buf[ptnsize];
		}

		if (fread(ptr_buf, 1, tbl_skip[*(ptr_buf - 1)], stream) < (size_t)tbl_skip[*(ptr_buf - 1)] && ferror(stream))
			err_printf(STR_FATAL_ERROR_IN_FREAD);
		ptr_buf  += tbl_skip[*(ptr_buf - 1)];
		file_pos += tbl_skip[*(ptr_buf - 1)];
	}

	/* Ȃꍇ́At@C|C^̈ʒuɖ߂ */
	if (!flag_success)
	{
		if (fseek(stream, file_offset, SEEK_SET))
			err_printf(STR_FATAL_ERROR_IN_FSEEK);
	}

	return !flag_success;
}
